namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsInversionView<MData, VData, EData, FData, readOnly>::NormalsInversionView(QMainWindow *parent) 
: View("normals inversion"), dock(new MeshNormalsInversionDock(parent))
{
  _normalsInverter = X3DProcessor::getInstanceOf<NormalsInverter<MData, VData, EData, FData, readOnly> >();
  _normalsFlipper = X3DProcessor::getInstanceOf<NormalsFlipper<MData, VData, EData, FData, readOnly> >();
  _normalsUpdater = X3DProcessor::getInstanceOf<NormalsUpdater<MData, VData, EData, FData, readOnly> >();
  _progressDock = new ProgressDock(parent, "work on progress...");
  
  RC::Route::connect(dock, &MeshNormalsInversionDock::invert, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::invert);
  RC::Route::connect(dock, &MeshNormalsInversionDock::makeCoherent, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::makeCoherent);
  RC::Route::connect(dock, &MeshNormalsInversionDock::computeNormals, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::computeNormals);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsInversionView<MData, VData, EData, FData, readOnly>::~NormalsInversionView()
{
  RC::Route::disconnect(dock, &MeshNormalsInversionDock::invert, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::invert);
  RC::Route::disconnect(dock, &MeshNormalsInversionDock::makeCoherent, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::makeCoherent);
  RC::Route::disconnect(dock, &MeshNormalsInversionDock::computeNormals, this, &NormalsInversionView<MData, VData, EData, FData, readOnly>::computeNormals);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::hide()
{
  View::hide();
  dock->hide();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::show()
{
  View::show();
  dock->show();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::invert()
{
  _normalsInverter->invert(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::makeCoherent()
{
  _normalsFlipper->flip(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::computeNormals()
{
  _normalsUpdater->updateNormals(getScene());
  notify();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInversionView<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  _normalsFlipper->setSelectedNodes(selectedNodes);
  _normalsInverter->setSelectedNodes(selectedNodes);
  _normalsUpdater->setSelectedNodes(selectedNodes);
}

}
}
