namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
StatisticsView<MData, VData, EData, FData, readOnly>::StatisticsView(QMainWindow *parent)
: View("statistics"), dock(new MeshStatisticsDock(parent))
{
  _statsCalculator = X3DProcessor::getInstanceOf<StatsCalculator<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
StatisticsView<MData, VData, EData, FData, readOnly>::~StatisticsView()
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>  
void StatisticsView<MData, VData, EData, FData, readOnly>::hide()
{
  View::hide();
  dock->hide();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatisticsView<MData, VData, EData, FData, readOnly>::show()
{
  View::show();
  dock->show();
}

template<class MData, class VData, class EData, class FData, bool readOnly> 
void StatisticsView<MData, VData, EData, FData, readOnly>::receiveNotification()
{
  _statsCalculator->calculate(getScene());
  MFNode selection = _statsCalculator->getSelectedNodes();
  
  if (selection.empty())
  {
    dock->setFaces(_statsCalculator->getModelFaces());
    dock->setVertices(_statsCalculator->getModelVertices());
    dock->setEdges(_statsCalculator->getModelEdges()); 
  }
  else
  {
    unsigned int faces = 0;
    unsigned int vertices = 0;
    unsigned int edges = 0;
    
    for (MFNode::const_iterator it = selection.begin(); it != selection.end(); ++it)
    {
      faces += _statsCalculator->getFacesOf(*it);
      vertices += _statsCalculator->getVerticesOf(*it);
      edges += _statsCalculator->getEdgesOf(*it);
    }
    
    dock->setFaces(faces);
    dock->setVertices(vertices);
    dock->setEdges(edges);
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatisticsView<MData, VData, EData, FData, readOnly>::display()
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatisticsView<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  _statsCalculator->setSelectedNodes(selectedNodes);
}

}
}
