#include "MESH_Shape.h"
#include "X3D_X3DAppearanceNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

Shape::Shape()
: X3DNode(), _appearance(0), _mesh(0)
{
  defineTypeName("Shape");
}

Shape::Shape(X3D::X3DAppearanceNode *appearance, X3DMeshNode *mesh)
: X3DNode(), _appearance(appearance), _mesh(0)
{
  defineTypeName("Shape");

  setMesh(static_cast<X3DMeshNode *>(mesh));
  setAppearance(static_cast<X3D::X3DAppearanceNode *>(appearance));
}

Shape::Shape(const Shape &S)
: X3DNode(S), _appearance(S._appearance), _mesh(0)
{
  setMesh(static_cast<X3DMeshNode *>(S._mesh));
  setAppearance(static_cast<X3D::X3DAppearanceNode *>(S._appearance));
}

SFNode Shape::clone() const
{
  return new Shape(*this);
}

Shape::~Shape()
{
  removeParentFromChild(this, _mesh);
}

void Shape::setMesh(X3DMeshNode *mesh)
{
  removeParentFromChild(this, _mesh);
  _mesh = mesh;
  addParentToChild(this, _mesh);
}

void Shape::setAppearance(X3D::X3DAppearanceNode *N)
{
  removeParentFromChild(this, _appearance);
  _appearance = N;
  addParentToChild(this, _appearance);
}

MFAbstractNode Shape::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (_mesh != 0)
    childrenList.push_back(_mesh);
  if (_appearance != 0)
    childrenList.push_back(_appearance);  
    
  return childrenList;
}

bool Shape::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMeshNode *>(N) != 0)
  {
    if (_mesh != 0)
      return false;
      
    setMesh(static_cast<X3DMeshNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3D::X3DAppearanceNode *>(N) != 0)
  {
    if (_appearance != 0)
      return false;
      
    setAppearance(static_cast<X3D::X3DAppearanceNode *>(N));
    return true;
  }
  
  if (N != 0)
    cx3d << "MESH::Shape::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "MESH::Shape::addChild : cannot add a NULL node!" << endl;
      
  return false;
}

bool Shape::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMeshNode *>(N) != 0)
  {
    setMesh(static_cast<X3DMeshNode *>(N));
    return true;
  }

  if (dynamic_cast<X3D::X3DAppearanceNode *>(N) != 0)
  {
    setAppearance(static_cast<X3D::X3DAppearanceNode *>(N));
    return true;
  }
  
  if (N != 0)
    cx3d << "MESH::Shape::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "MESH::Shape::setChild : cannot add a NULL node!" << endl;
      
  return false;
}

bool Shape::removeChild(const SFAbstractNode &N)
{
  if (_mesh == N)
  {
    setMesh(0);
    return true;
  } 

  if (_appearance == N)
  {
    setAppearance(0);
    return true;
  } 
  
  return false;
}

}
}
