#include "X3D_DirectionalLight.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

DirectionalLight::DirectionalLight()
  : X3DLightNode(),
    _direction(SFVec3f(0.0f,0.0f,-1.0f))
{
  defineTypeName("DirectionalLight", "Lighting");
}

DirectionalLight::DirectionalLight(const SFFloat       &ambientIntensity,
                                   const SFColor &color,
                                   const SFVec3f &direction,
                                   const SFFloat       &intensity,
                                   const SFBool        &on)
  : X3DLightNode(ambientIntensity,color,intensity,on),
    _direction(direction)
{
  defineTypeName("DirectionalLight", "Lighting");
}

DirectionalLight::DirectionalLight(const DirectionalLight &N)
  : X3DLightNode(N),
    _direction(N._direction)
{
}

SFAbstractNode DirectionalLight::clone() const
{
  return new DirectionalLight(*this);
}

DirectionalLight::~DirectionalLight()
{
}

void DirectionalLight::setDirection(const SFVec3f &direction)
{
  _direction = direction;
}

void DirectionalLight::load(const X3DFileElement *element)
{
  X3DLightNode::load(element);

  int index;
  index = element->getIndexAttribute("direction");
  if (index != -1)
    _direction = SFVec3f(element->getAttribute(index));
}

SFString &DirectionalLight::write(SFString &output) const
{
  X3DLightNode::write(output);
  
  if (_direction != SFVec3f(0.0f,0.0f,-1.0f))  
    output += " direction=\"" + toSFString(_direction) + "\"";
    
  return output;  
}

}
}
