#include "X3D_Appearance.h"
#include "X3D_X3DMaterialNode.h"
#include "X3D_X3DTextureNode.h"
#include "X3D_X3DTextureTransformNode.h"
#include "X3D_LineProperties.h"
#include "X3D_FillProperties.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

Appearance::Appearance()
: X3DAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTypeName("Appearance", "Shape");
}

Appearance::Appearance(X3DMaterialNode *material, X3DTextureNode *texture, X3DTextureTransformNode *textureTransform, LineProperties *lineProperties, FillProperties *fillProperties)
: X3DAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTypeName("Appearance", "Shape");
  
  setMaterial(static_cast<X3DMaterialNode *>(material));
  setTexture(static_cast<X3DTextureNode *>(texture));
  setTextureTransform(static_cast<X3DTextureTransformNode *>(textureTransform));
  setLineProperties(static_cast<LineProperties *>(lineProperties));
  setFillProperties(static_cast<FillProperties *>(fillProperties));
}

Appearance::Appearance(const Appearance &A)
: X3DAppearanceNode(A), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  setMaterial(static_cast<X3DMaterialNode *>(A._material));
  setTexture(static_cast<X3DTextureNode *>(A._texture));
  setTextureTransform(static_cast<X3DTextureTransformNode *>(A._textureTransform));
  setLineProperties(static_cast<LineProperties *>(A._lineProperties));
  setFillProperties(static_cast<FillProperties *>(A._fillProperties));
}

SFAbstractNode Appearance::clone() const
{
  return new Appearance(*this);
}

Appearance::~Appearance()
{
  removeParentFromChild(this, _material);
  removeParentFromChild(this, _texture);
  removeParentFromChild(this, _textureTransform);
  removeParentFromChild(this, _lineProperties);
  removeParentFromChild(this, _fillProperties);
}

void Appearance::setMaterial(X3DMaterialNode *material)
{
  removeParentFromChild(this, _material);
  _material = material;
  addParentToChild(this, _material);
}

void Appearance::setTexture(X3DTextureNode *texture)
{
  removeParentFromChild(this, _texture);
  _texture = texture;
  addParentToChild(this, _texture);
}

void Appearance::setTextureTransform(X3DTextureTransformNode *textureTransform)
{
  removeParentFromChild(this, _textureTransform);
  _textureTransform = textureTransform;
  addParentToChild(this, _textureTransform);
}

void Appearance::setLineProperties(LineProperties *lineProperties)
{
  removeParentFromChild(this, _lineProperties);
  _lineProperties = lineProperties;
  addParentToChild(this, _lineProperties);
}

void Appearance::setFillProperties(FillProperties *fillProperties)
{
  removeParentFromChild(this, _fillProperties);
  _fillProperties = fillProperties;
  addParentToChild(this, _fillProperties);
}

MFAbstractNode Appearance::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (_material != 0)
    childrenList.push_back(_material);
  if (_texture != 0)    
    childrenList.push_back(_texture);
  if (_textureTransform != 0)    
    childrenList.push_back(_textureTransform);
  if (_lineProperties != 0)    
    childrenList.push_back(_lineProperties);
  if (_fillProperties != 0)    
    childrenList.push_back(_fillProperties);
    
  return childrenList; 
}

bool Appearance::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    if (_material != 0)
      return false;
      
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    if (_texture != 0)
      return false;
      
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    if (_textureTransform != 0)
      return false;
      
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    if (_lineProperties != 0)
      return false;
      
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    if (_fillProperties != 0)
      return false;
      
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::Appearance::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::Appearance::addChild : cannot add a NULL node!" << endl;
  return false;
}

bool Appearance::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::Appearance::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::Appearance::setChild : cannot add a NULL node!" << endl;
    
  return false;
}

bool Appearance::removeChild(const SFAbstractNode &N)
{
  if (_material == N)
  {
    setMaterial(0);
    return true;
  }
  
  if (_texture == N)
  {
    setTexture(0);
    return true; 
  }
  
  if (_textureTransform == N)
  {
    setTextureTransform(0);    
    return true;  
  }
  
  if (_lineProperties == N)
  {
    setLineProperties(0);
    return true;
  }
  
  if (_fillProperties == N)
  {
    setFillProperties(0);
    return true; 
  }
  
  return false;
}

}
}
