#include "GL_RendererShapeVisitor.h"
#include "GL_Material.h"
#include "GL_X3DShapeNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

RendererShapeVisitor::RendererShapeVisitor()
: ShapeVisitor()
{
  // Enter functions.
  defineEnterFunction(&RendererShapeVisitor::enterMaterial);
  defineEnterFunction(&RendererShapeVisitor::enterX3DShapeNode);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<RendererStateVariables>();
}

RendererShapeVisitor::~RendererShapeVisitor()
{
}

void RendererShapeVisitor::enterMaterial(Material *M) const
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, &M->getDiffuseColor().front());
  glMaterialfv(GL_FRONT, GL_AMBIENT, &M->getAmbientColor().front());
  glMaterialfv(GL_FRONT, GL_EMISSION, &M->getEmissiveColor().front());
  SFFloat s = M->getShininess();
  glMaterialfv(GL_FRONT, GL_SHININESS, &s); 
  glMaterialfv(GL_FRONT, GL_SPECULAR, &M->getSpecularColor().front());
}

void RendererShapeVisitor::enterX3DShapeNode(X3DShapeNode *N) const
{
  if (stateVariables->getSelectionMode())
  {
    stateVariables->storeShape(N);
    glLoadName(stateVariables->getCurrentId());
//    cout << "loading name " << stateVariables->getCurrentId() << endl;
  }  
}

}
}
